/**
 * A directive used to map template attributes, properties and events to element
 */
import { noChange } from 'lit';
import { Directive, PartType, directive } from 'lit/directive.js';
var MAP_TYPE;
(function (MAP_TYPE) {
    MAP_TYPE[MAP_TYPE["ATTRIBUTE"] = 0] = "ATTRIBUTE";
    MAP_TYPE[MAP_TYPE["PROPERTY"] = 1] = "PROPERTY";
    MAP_TYPE[MAP_TYPE["LISTENER"] = 2] = "LISTENER";
})(MAP_TYPE || (MAP_TYPE = {}));
/**
 * Get map type as follows
 * - starts from '.' is a property
 * - starts from '@' is an event listener
 * - otherwise it is an attribute
 * @param key The key to check
 * @returns 0 for `attribute`; 1 for `property`; 2 for `listener`
 */
const getMapType = (key) => {
    const startsWith = key[0];
    if (startsWith === '.') {
        return MAP_TYPE.PROPERTY;
    }
    if (startsWith === '@') {
        return MAP_TYPE.LISTENER;
    }
    return MAP_TYPE.ATTRIBUTE;
};
/**
 * Get the name of map property that excludes '.' and '@' from the name
 * @param key The key to get map name for
 * @returns name Map name
 */
const getMapName = (key) => {
    const type = getMapType(key);
    if (type === MAP_TYPE.PROPERTY || type === MAP_TYPE.LISTENER) {
        return key.substring(1);
    }
    return key;
};
/**
 * Apply attribute, property or event listener to an element
 * @param element Element
 * @param name Name of attribute, property (starts with `.`) or event listener (starts with `@`)
 * @param value New Value
 * @param oldValue Old value
 * @returns {void}
 */
const setMapped = (element, name, value, oldValue) => {
    const type = getMapType(name);
    name = getMapName(name);
    switch (type) {
        case MAP_TYPE.LISTENER:
            if (typeof oldValue === 'function') {
                element.removeEventListener(name, oldValue);
            }
            if (typeof value === 'function') {
                element.addEventListener(name, value);
            }
            break;
        case MAP_TYPE.PROPERTY:
            if (value === undefined) {
                delete element[name];
            }
            else {
                element[name] = value;
            }
            break;
        case MAP_TYPE.ATTRIBUTE:
        default:
            // remove with undefined to comply with property
            if (value === null || value === undefined || value === false) {
                element.removeAttribute(name);
            }
            else if (value === true) {
                element.setAttribute(name, '');
            }
            else {
                element.setAttribute(name, String(value));
            }
            break;
    }
};
class TemplateMapDirective extends Directive {
    constructor(partInfo) {
        super(partInfo);
        this.valueMap = {}; // used to cache values
        if (partInfo.type !== PartType.ELEMENT) {
            throw new Error('The `TemplateMap` directive must be used with element');
        }
    }
    /**
     * @param part Element part
     * @param attributeMap Attribute map
     * @returns noChange
     */
    update(part, [attributeMap]) {
        const newValueMap = {};
        const element = part.element;
        for (const name in attributeMap) {
            const value = attributeMap[name];
            let scopedValue = value;
            let oldValue;
            let oldScopedValue;
            if (this.valueMap[name]) {
                oldValue = this.valueMap[name].value;
                oldScopedValue = scopedValue = this.valueMap[name].scopedValue;
            }
            if (oldValue !== value) {
                // always assume that if `function` is passed it is an event listener
                scopedValue = typeof value === 'function' ? value.bind(part.options?.host || element) : value;
                setMapped(element, name, scopedValue, oldScopedValue);
            }
            newValueMap[name] = { value, scopedValue };
            delete this.valueMap[name];
        }
        // Delete old cached values
        for (const name in this.valueMap) {
            setMapped(element, name, undefined, this.valueMap[name].scopedValue);
        }
        this.valueMap = newValueMap;
        return this.render();
    }
    /**
     * @returns noChange
     */
    render() {
        return noChange;
    }
}
/**
 * A directive to apply attributes to template element
 * @param template Template map with the following syntax:
 * {
 *   'label': 'value', // set attribute with attribute value, e.g. `<input label="value">`
 *   'required': true, // set empty attribute, e.g. `<input required>`
 *   'error': null | undefined | false, // remove attribute `error`
 *   '.prop': 'value', // set property `.prop='value'`
 *   '.warning': undefined, // remove property `warning`
 *   '@listener': this.eventListener, // set event listener via element.addEventListener('listener', this.eventListener.bind(this)),
 *   '@changed': undefined // remove event listener `changed`
 * }
 * @returns noChange directive result
 */
const templateMap = directive(TemplateMapDirective);
export { templateMap };
