import { LitElement } from 'lit';
import type { CSSValue } from '../types/base';
import { BasicElementSymbol } from '../utils/helpers.js';
/**
 * Basic element base class.
 * Usually used for creating low-level elements.
 *
 * @fires focused-changed Fired when `focused` property changes
 */
export declare abstract class BasicElement extends LitElement {
    /**
     * Creates and registers instance of Element.
     */
    constructor();
    /**
     * Apply theme styles
     * @param theme Theme CSS
     * @returns {void}
     */
    static applyThemeStyles(theme: string): void;
    /**
     * Set the default tabindex of element
     * null - for unset
     * -1 - can be focused; but cannot be tabbed
     * 0 - can be focused; can be tabbed; the order is based on DOM order
     * >= 1 - can be focused; can be tabbed; the order is based on tabindex (anti-pattern)
     */
    protected readonly defaultTabIndex: number | null;
    /**
     * Element's role attribute for accessibility
     */
    protected readonly defaultRole: string | null;
    /**
     * False to not delegate the focus by default
     */
    readonly delegatesFocus: boolean;
    /**
     * The element should be automatically focused after added to the DOM.
     * Only one element can be auto-focused at any time.
     */
    autofocus: boolean;
    /**
     * Get focused state of an element:
     * @readonly
     */
    get focused(): boolean;
    /**
     * Gets any defined css variables by name/key
     * @param options options list of variables and fallbacks
     * @returns value of the css variable, or, fallback if specified, when a a variable is null.
     * @example
     * this.getComputedVariable('--valid-name'); // return value of the --valid-name
     * this.getComputedVariable('--invalid-name', '10px'); // return fallback value 10px
     */
    protected getComputedVariable(...options: CSSValue[]): string;
    /**
     * Updates, adds, or removes a css variable from the element.
     * @param key css variable key
     * @param value css variable value
     * @returns {void}
     */
    protected updateVariable(key: string, value: CSSValue | null | undefined): void;
    /**
     * Dispatch property change event when the property's value has been changed.
     * Event name is transformed to hyphen case, e.g. myProperty -> my-property-changed.
     * Event details contain the new value.
     * @param name Property name
     * @param value New value
     * @param [cancelable=false] Set to true if the event can be cancelled
     * @returns false if the event is prevented
     */
    protected notifyPropertyChange(name: string, value: unknown, cancelable?: boolean): boolean;
    /**
     * Dispatch input event when the property's value is being input.
     * Event name is transformed to hyphen case, e.g. myProperty -> my-property-input.
     * Except for value property it will transformed to input instead of value-input.
     * Event details contain the new value.
     * @param name Property name
     * @param value New value
     * @param [cancelable=false] Set to true if the event can be cancelled
     * @returns false if the event is prevented
     */
    protected notifyPropertyInput(name: string, value: unknown, cancelable?: boolean): boolean;
    /**
     * Registers the connection to the DOM
     * @returns {void}
     */
    connectedCallback(): void;
    /**
     * Registers the disconnection from the DOM
     * @returns {void}
     */
    disconnectedCallback(): void;
    /**
     * Check to see if the element if tabbable.
     * Element is tabbable if it has tabindex >=0
     * and is visible on the screen.
     */
    get tabbable(): boolean;
    /**
     * Get a sorted collection of nodes that can be tabbed through.
     */
    get tabbableElements(): HTMLElement[];
    /**
     * A symbol to check if an element is BasicElement
     */
    static readonly [BasicElementSymbol]: symbol;
    /**
     * Placeholder for getting an element's version number
     * @returns version number
     */
    static get version(): string;
}
