import type { PropertyValues } from 'lit';
import type { ControlProperties as IControlProperties } from '../interfaces/ControlProperties';
import { BasicElement } from './BasicElement.js';
/**
 * Control element base class.
 * Usually used for creating form-style elements.
 */
export declare abstract class ControlElement extends BasicElement implements IControlProperties {
    /**
     * All control element by default need to be focusable
     */
    protected readonly defaultTabIndex: number | null;
    /**
     * Control element should delegate focus by default
     */
    readonly delegatesFocus: boolean;
    /**
     * Name of the element.
     */
    name: string;
    /**
     * Indicates whether the user can interact with the element.
     */
    disabled: boolean;
    /**
     * Value of the element
     * @param value Element value
     */
    set value(value: string);
    get value(): string;
    /**
     * Indicates whether the user can interact and still show value with the element.
     */
    readonly: boolean;
    /**
     * Value of the `tabindex` before focusing was disabled.
     */
    private oldTabIndex;
    /**
     * Internal value of the element.
     * Controlled by public setter and getter
     */
    private _value;
    /**
     * Get a sorted collection of nodes that can be tabbed through if component not disabled.
     */
    get tabbableElements(): HTMLElement[];
    /**
     * @override
     * @returns {void}
     */
    protected update(changedProperties: PropertyValues): void;
    /**
     * Update disabled state if detect value changed
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    protected disableChanged(changedProperties: PropertyValues): void;
    /**
     * Re-enables focus, if the focus has been previously
     * disabled by running disableFocus.
     * @returns {void}
     */
    protected enableFocus(): void;
    /**
     * Disables the ability to focus and tab on element.
     * @returns {void}
     */
    protected disableFocus(): void;
    /**
     * Cast value to string
     * @param value Value that is not string, but
     * which may be set by app developer, e.g. number or date
     * @returns string representation of the value
     */
    protected castValue(value: unknown): string;
    /**
     * Used to show a warning when the value does not pass the validation
     * @param value that is invalid
     * @returns {void}
     */
    protected warnInvalidValue(value: string): void;
    /**
     * Validate that the value confirms the control type
     * The validation resets value.
     * For input validation use lazy validation.
     * @param value Value to check
     * @returns false if value is invalid
     */
    protected isValidValue(value: string): boolean;
    /**
     * On *user-interaction* set the value and notify.
     * @param value New value
     * @returns {void}
     */
    protected setValueAndNotify(value: string): void;
    /**
     * Resets the element value to its initial setting
     * @returns Whether the value has changed
     */
    reset(): boolean;
}
