import { __decorate } from "tslib";
import { property } from '../decorators/property.js';
import { WarningNotice } from '../notices/WarningNotice.js';
import { BasicElement } from './BasicElement.js';
/**
 * Control element base class.
 * Usually used for creating form-style elements.
 */
export class ControlElement extends BasicElement {
    constructor() {
        super(...arguments);
        /**
         * All control element by default need to be focusable
         */
        this.defaultTabIndex = 0;
        /**
         * Control element should delegate focus by default
         */
        this.delegatesFocus = true;
        /**
         * Name of the element.
         */
        this.name = '';
        /**
         * Indicates whether the user can interact with the element.
         */
        this.disabled = false;
        /**
         * Indicates whether the user can interact and still show value with the element.
         */
        this.readonly = false;
        /**
         * Value of the `tabindex` before focusing was disabled.
         */
        this.oldTabIndex = null;
        /**
         * Internal value of the element.
         * Controlled by public setter and getter
         */
        this._value = '';
    }
    /**
     * Value of the element
     * @param value Element value
     */
    set value(value) {
        const oldValue = this._value;
        value = this.castValue(value);
        if (!this.isValidValue(value)) {
            this.warnInvalidValue(value);
            value = '';
        }
        if (oldValue !== value) {
            this._value = value;
            this.requestUpdate('value', oldValue);
        }
    }
    get value() {
        return this._value;
    }
    /**
     * Get a sorted collection of nodes that can be tabbed through if component not disabled.
     */
    get tabbableElements() {
        return this.disabled ? [] : super.tabbableElements;
    }
    /**
     * @override
     * @returns {void}
     */
    update(changedProperties) {
        if (changedProperties.has('disabled')) {
            this.disableChanged(changedProperties);
        }
        super.update(changedProperties);
    }
    /**
     * Update disabled state if detect value changed
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    disableChanged(changedProperties) {
        if (this.disabled) {
            this.disableFocus();
            this.setAttribute('aria-disabled', 'true');
        }
        else if (changedProperties.get('disabled') === true) {
            /* re-enable only if disabled changed from true to false */
            this.enableFocus();
            this.removeAttribute('aria-disabled');
        }
    }
    /**
     * Re-enables focus, if the focus has been previously
     * disabled by running disableFocus.
     * @returns {void}
     */
    enableFocus() {
        this.style.removeProperty('pointer-events');
        if (this.oldTabIndex !== null && this.tabIndex === -1) {
            this.tabIndex = this.oldTabIndex;
        }
        this.oldTabIndex = null;
    }
    /**
     * Disables the ability to focus and tab on element.
     * @returns {void}
     */
    disableFocus() {
        this.style.setProperty('pointer-events', 'none');
        if (this.hasAttribute('tabindex') && this.tabIndex >= 0) {
            this.oldTabIndex = this.tabIndex;
            this.tabIndex = -1;
        }
        this.hasAttribute('focused') && this.blur();
    }
    /**
     * Cast value to string
     * @param value Value that is not string, but
     * which may be set by app developer, e.g. number or date
     * @returns string representation of the value
     */
    castValue(value) {
        if (typeof value === 'string') {
            return value;
        }
        if (value === null) {
            return '';
        }
        return String(value);
    }
    /**
     * Used to show a warning when the value does not pass the validation
     * @param value that is invalid
     * @returns {void}
     */
    warnInvalidValue(value) {
        new WarningNotice(`The specified value "${value}" is not valid.`).show();
    }
    /**
     * Validate that the value confirms the control type
     * The validation resets value.
     * For input validation use lazy validation.
     * @param value Value to check
     * @returns false if value is invalid
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    isValidValue(value) {
        return true;
    }
    /**
     * On *user-interaction* set the value and notify.
     * @param value New value
     * @returns {void}
     */
    setValueAndNotify(value) {
        if (this.value !== value) {
            this.value = value;
            this.notifyPropertyChange('value', value);
        }
    }
    /**
     * Resets the element value to its initial setting
     * @returns Whether the value has changed
     */
    reset() {
        const initialValue = this.getAttribute('value') || '';
        const currentValue = this.value;
        if (currentValue !== initialValue) {
            this.value = initialValue;
            return true;
        }
        return false;
    }
}
__decorate([
    property({ type: String })
], ControlElement.prototype, "name", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], ControlElement.prototype, "disabled", void 0);
__decorate([
    property({ type: String })
], ControlElement.prototype, "value", null);
__decorate([
    property({ type: Boolean, reflect: true })
], ControlElement.prototype, "readonly", void 0);
