import { PropertyValues, TemplateResult, nothing } from 'lit';
import { TemplateMap } from '../directives/template-map.js';
import { TapEvent } from '../events/TapEvent.js';
import { ControlElement } from './ControlElement.js';
type SelectionDirection = 'forward' | 'backward' | 'none';
/**
 * Form Field Element base class.
 * Used for form field type controls.
 * Adds support for assistive technologies, `error` and `warning` states
 */
export declare abstract class FormFieldElement extends ControlElement {
    /**
     * Returns a list of attributes corresponding to the registered properties.
     */
    static get observedAttributes(): string[];
    /**
     * Set state to error
     */
    error: boolean;
    /**
     * Set state to warning
     */
    warning: boolean;
    /**
     * Set placeholder text
     */
    placeholder: string;
    /**
     * Disables all other states and border/background styles.
     * Use with advanced composite elements requiring e.g. multi selection in
     * combo-box when parent container handles element states.
     */
    transparent: boolean;
    /**
     * Show clears button
     */
    clears: boolean;
    /**
     * Internal reference to clears button
     */
    private clearsButtonRef;
    /**
     * Get an input element
     */
    protected get clearsButton(): HTMLElement | undefined;
    /**
     * A reference to input element
     */
    private inputElRef;
    /**
     * Get an input element
     */
    protected get inputElement(): HTMLInputElement | undefined;
    /**
     * Used to get `aria-label` field for internal <input>
     * Calculated from `aria-label`, `aria-labelledby` and `label[for="<element.id>"]`
     */
    protected inputAriaLabel: string | null;
    /**
     * Used to get `aria-description` field for internal <input>
     * Calculated from `aria-description` or `aria-describedby`
     */
    protected inputAriaDescription: string | null;
    /**
     * Used to get `aria-required` field for internal <input>
     * Calculated from `aria-required`
     */
    protected inputAriaRequired: boolean;
    /**
     * Get native input value
     * @returns string of input value
     */
    protected get inputValue(): string;
    /**
     * Set native input value
     * @param value input's value
     */
    protected set inputValue(value: string);
    /**
     * Returns true if an input element contains valid data.
     * @returns true if input is valid
     */
    checkValidity(): boolean;
    /**
     * Validate input. Mark as error if input is invalid
     * @returns false if there is an error
     */
    reportValidity(): boolean;
    /**
     * Synchronizes attribute value
     * @param name attribute name
     * @param oldValue old attribute value
     * @param newValue new attribute value
     * @returns {void}
     */
    attributeChangedCallback(name: string, oldValue: string | null, newValue: string | null): void;
    /**
     * Compute property values that depend on other properties
     * and are used in the rest of the update process.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    willUpdate(changedProperties: PropertyValues): void;
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @returns {void}
     */
    protected firstUpdated(changedProperties: PropertyValues): void;
    /**
     * Handles the focus event
     * @param event Focus event
     * @returns {void}
     */
    protected onFocus(event: FocusEvent): void;
    /**
     * Set inherited aria properties
     * @returns {void}
     */
    protected setInheritedAria(): void;
    /**
     * Notify error if it has changed
     * @param hasError true if the element has an error
     * @returns {void}
     */
    protected notifyErrorChange(hasError: boolean): void;
    /**
     * Runs on input element `input` event
     * @param event `input` event
     * @returns {void}
     */
    protected onInputInput(event: InputEvent): void;
    /**
     * Runs on input element `change` event
     * @param event `change` event
     * @returns {void}
     */
    protected onInputChange(event: InputEvent): void;
    /**
     * Decorate `<input>` element with common properties:
     * aria-label - calculated from `aria-label`, `aria-labelledby` and `label[for="<element.id>"]`
     * aria-description - calculated from `aria-description` or `aria-describedby`
     * aria-invalid="true|null" - calculated on based on `error` state
     * aria-required="true|null" - calculated on based on `aria-required`
     * placeholder - placeholder text
     * readonly - indicates whether the user can interact and still show value with the element
     * disabled - indicates whether the user can interact with the element
     * autocomplete="off" - always off as is not supported within shadow root
     * @input - Listener for `input` event. Runs `this.onInputInput`
     * @change - Listener for `change` event. Runs `this.onInputChange`
     * @returns template map
     */
    protected get decorateInputMap(): TemplateMap;
    /**
     * Renders input element
     * @returns {void}
     */
    protected renderInput(): TemplateResult;
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    protected render(): TemplateResult;
    /**
     * Get selection start index
     */
    get selectionStart(): number | null;
    /**
     * Set selection start index
     * @param index Start index
     */
    set selectionStart(index: number | null);
    /**
     * Get selection end index
     */
    get selectionEnd(): number | null;
    /**
     * Set selection end index
     * @param index End index
     */
    set selectionEnd(index: number | null);
    /**
     * Gets the direction in which selection occurred
     */
    get selectionDirection(): SelectionDirection | null;
    /**
     * Sets the direction in which selection occurred
     * @param direction Selection direction
     */
    set selectionDirection(direction: SelectionDirection | null);
    /**
     * Select the contents of input
     * @returns void
     */
    select(): void;
    /**
     * Set the selection range
     * @param startSelection Start of selection
     * @param endSelection End of the selection
     * @param [selectionDirection=none] A string indicating the direction in which the selection is considered to have been performed.
     * @returns {void}
     */
    setSelectionRange(startSelection: number | null, endSelection: number | null, selectionDirection?: SelectionDirection): void;
    /**
     * Template for clear button
     * Rendered when `clears` attribute is set
     * @returns Popup template or undefined
     */
    protected get clearButtonTemplate(): TemplateResult | typeof nothing;
    /**
     * Run when tap event happens on clears button and fire value-changed event.
     * @param event tap event
     * @returns {void}
     */
    protected onClearsButtonTap(event: TapEvent): void;
    /**
     * Returns a condition to show clear button
     * @returns True if clear button will show
     */
    protected get hasClear(): boolean;
}
export {};
