import { __decorate } from "tslib";
import { html, nothing } from 'lit';
import { description as inputDescription } from '@refinitiv-ui/utils/accessibility/description.js';
import { label as inputLabel } from '@refinitiv-ui/utils/accessibility/label.js';
import { required as inputRequired } from '@refinitiv-ui/utils/accessibility/required.js';
import { property } from '../decorators/property.js';
import { state } from '../decorators/state.js';
import { createRef, ref } from '../directives/ref.js';
import { templateMap } from '../directives/template-map.js';
import { ControlElement } from './ControlElement.js';
const AriaLabelKey = Symbol('aria-label-key');
const AriaDescriptionKey = Symbol('aria-description-key');
const AriaRequiredKey = Symbol('aria-required-key');
/**
 * Observed attributes that cause `aria-label` to be recalculated
 */
const ObservedAriaLabel = ['aria-label', 'aria-labelledby', 'id'];
/**
 * Observed attributes that cause `aria-description` to be recalculated
 */
const ObservedAriaDescription = ['aria-description', 'aria-describedby'];
/**
 * Observed attributes that cause `aria-required` to be recalculated
 */
const ObservedAriaRequired = ['aria-required'];
/**
 * Form Field Element base class.
 * Used for form field type controls.
 * Adds support for assistive technologies, `error` and `warning` states
 */
export class FormFieldElement extends ControlElement {
    constructor() {
        super(...arguments);
        /**
         * Set state to error
         */
        this.error = false;
        /**
         * Set state to warning
         */
        this.warning = false;
        /**
         * Set placeholder text
         */
        this.placeholder = '';
        /**
         * Disables all other states and border/background styles.
         * Use with advanced composite elements requiring e.g. multi selection in
         * combo-box when parent container handles element states.
         */
        this.transparent = false;
        /**
         * Show clears button
         */
        this.clears = false;
        /**
         * Internal reference to clears button
         */
        this.clearsButtonRef = createRef();
        /**
         * A reference to input element
         */
        this.inputElRef = createRef();
        /**
         * Used to get `aria-label` field for internal <input>
         * Calculated from `aria-label`, `aria-labelledby` and `label[for="<element.id>"]`
         */
        this.inputAriaLabel = null;
        /**
         * Used to get `aria-description` field for internal <input>
         * Calculated from `aria-description` or `aria-describedby`
         */
        this.inputAriaDescription = null;
        /**
         * Used to get `aria-required` field for internal <input>
         * Calculated from `aria-required`
         */
        this.inputAriaRequired = false;
    }
    /**
     * Returns a list of attributes corresponding to the registered properties.
     */
    static get observedAttributes() {
        return Array.from(new Set([
            ...super.observedAttributes,
            ...ObservedAriaLabel,
            ...ObservedAriaDescription,
            ...ObservedAriaRequired
        ]));
    }
    /**
     * Get an input element
     */
    get clearsButton() {
        return this.clearsButtonRef.value;
    }
    /**
     * Get an input element
     */
    get inputElement() {
        return this.inputElRef.value;
    }
    /**
     * Get native input value
     * @returns string of input value
     */
    get inputValue() {
        return this.inputElement ? this.inputElement.value : '';
    }
    /**
     * Set native input value
     * @param value input's value
     */
    set inputValue(value) {
        if (this.inputElement) {
            this.inputElement.value = value;
        }
    }
    /**
     * Returns true if an input element contains valid data.
     * @returns true if input is valid
     */
    checkValidity() {
        return this.inputElement ? this.inputElement.checkValidity() : false;
    }
    /**
     * Validate input. Mark as error if input is invalid
     * @returns false if there is an error
     */
    reportValidity() {
        const hasError = !this.checkValidity();
        this.notifyErrorChange(hasError);
        return !hasError;
    }
    /**
     * Synchronizes attribute value
     * @param name attribute name
     * @param oldValue old attribute value
     * @param newValue new attribute value
     * @returns {void}
     */
    attributeChangedCallback(name, oldValue, newValue) {
        super.attributeChangedCallback(name, oldValue, newValue);
        // Do not use lit properties, as these may change the way how native aria works.
        // Instead silently listen for changes and hook into update lifecycle
        // Future AOM should reflect properties to attributes, so this code should still work
        if (ObservedAriaLabel.includes(name)) {
            this.requestUpdate(AriaLabelKey, Symbol());
        }
        if (ObservedAriaDescription.includes(name)) {
            this.requestUpdate(AriaDescriptionKey, Symbol());
        }
        if (ObservedAriaRequired.includes(name)) {
            this.requestUpdate(AriaRequiredKey, Symbol());
        }
    }
    /**
     * Compute property values that depend on other properties
     * and are used in the rest of the update process.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (changedProperties.has(AriaLabelKey)) {
            this.inputAriaLabel = inputLabel(this);
        }
        if (changedProperties.has(AriaRequiredKey)) {
            this.inputAriaRequired = inputRequired(this);
        }
        // Description is traditionally changed with error, so recalculate on error changed as well
        if (changedProperties.has(AriaDescriptionKey) ||
            (changedProperties.get('error') !== undefined && changedProperties.has('error'))) {
            this.inputAriaDescription = inputDescription(this);
        }
    }
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('focus', this.onFocus);
    }
    /**
     * Handles the focus event
     * @param event Focus event
     * @returns {void}
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    onFocus(event) {
        // Always recalculate inherited arias on focus and these may change from last time
        this.setInheritedAria();
    }
    /**
     * Set inherited aria properties
     * @returns {void}
     */
    setInheritedAria() {
        this.inputAriaLabel = inputLabel(this);
        this.inputAriaDescription = inputDescription(this);
        // no need for required or error here, as it is sufficient to recalculate these on attributeChanged
    }
    /**
     * Notify error if it has changed
     * @param hasError true if the element has an error
     * @returns {void}
     */
    notifyErrorChange(hasError) {
        if (this.error !== hasError) {
            this.error = hasError;
            this.notifyPropertyChange('error', this.error);
        }
    }
    /**
     * Runs on input element `input` event
     * @param event `input` event
     * @returns {void}
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    onInputInput(event) {
        // nothing
    }
    /**
     * Runs on input element `change` event
     * @param event `change` event
     * @returns {void}
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    onInputChange(event) {
        // nothing
    }
    /**
     * Decorate `<input>` element with common properties:
     * aria-label - calculated from `aria-label`, `aria-labelledby` and `label[for="<element.id>"]`
     * aria-description - calculated from `aria-description` or `aria-describedby`
     * aria-invalid="true|null" - calculated on based on `error` state
     * aria-required="true|null" - calculated on based on `aria-required`
     * placeholder - placeholder text
     * readonly - indicates whether the user can interact and still show value with the element
     * disabled - indicates whether the user can interact with the element
     * autocomplete="off" - always off as is not supported within shadow root
     * @input - Listener for `input` event. Runs `this.onInputInput`
     * @change - Listener for `change` event. Runs `this.onInputChange`
     * @returns template map
     */
    get decorateInputMap() {
        return {
            'aria-label': this.inputAriaLabel,
            'aria-description': this.inputAriaDescription,
            'aria-invalid': this.error ? 'true' : null,
            'aria-required': this.inputAriaRequired ? 'true' : null,
            placeholder: this.placeholder || null,
            readonly: this.readonly,
            disabled: this.disabled,
            autocomplete: 'off',
            '@input': this.onInputInput,
            '@change': this.onInputChange
        };
    }
    /**
     * Renders input element
     * @returns {void}
     */
    renderInput() {
        return html `<input ${templateMap(this.decorateInputMap)} ${ref(this.inputElRef)} />`;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `${this.renderInput()}`;
    }
    /**
     * Get selection start index
     */
    get selectionStart() {
        return this.inputElement ? this.inputElement.selectionStart : null;
    }
    /**
     * Set selection start index
     * @param index Start index
     */
    set selectionStart(index) {
        if (this.inputElement) {
            this.inputElement.selectionStart = index;
        }
    }
    /**
     * Get selection end index
     */
    get selectionEnd() {
        return this.inputElement ? this.inputElement.selectionEnd : null;
    }
    /**
     * Set selection end index
     * @param index End index
     */
    set selectionEnd(index) {
        if (this.inputElement) {
            this.inputElement.selectionEnd = index;
        }
    }
    /**
     * Gets the direction in which selection occurred
     */
    get selectionDirection() {
        return this.inputElement ? this.inputElement.selectionDirection : null;
    }
    /**
     * Sets the direction in which selection occurred
     * @param direction Selection direction
     */
    set selectionDirection(direction) {
        if (this.inputElement) {
            this.inputElement.selectionDirection = direction;
        }
    }
    /**
     * Select the contents of input
     * @returns void
     */
    select() {
        if (!this.disabled && this.inputElement) {
            this.inputElement.select();
        }
    }
    /**
     * Set the selection range
     * @param startSelection Start of selection
     * @param endSelection End of the selection
     * @param [selectionDirection=none] A string indicating the direction in which the selection is considered to have been performed.
     * @returns {void}
     */
    setSelectionRange(startSelection, endSelection, selectionDirection) {
        this.inputElement?.setSelectionRange(startSelection, endSelection, selectionDirection);
    }
    /**
     * Template for clear button
     * Rendered when `clears` attribute is set
     * @returns Popup template or undefined
     */
    get clearButtonTemplate() {
        if (this.hasClear) {
            return html `
        <div
          aria-hidden="true"
          ${ref(this.clearsButtonRef)}
          id="clears-button"
          part="button button-clear"
          @tap="${this.onClearsButtonTap}"
        >
          <ef-icon part="icon icon-clear" icon="cross"></ef-icon>
        </div>
      `;
        }
        return nothing;
    }
    /**
     * Run when tap event happens on clears button and fire value-changed event.
     * @param event tap event
     * @returns {void}
     */
    onClearsButtonTap(event) {
        this.setValueAndNotify('');
        // Some form control elements, such as Tree Select, display/hide an overlay upon selection.
        // PreventDefault() ensures that selecting the clear button would not change overlay state.
        event.preventDefault();
    }
    /**
     * Returns a condition to show clear button
     * @returns True if clear button will show
     */
    get hasClear() {
        const editable = !(this.readonly || this.disabled); // shouldn't display clear if disabled or readonly
        return this.clears && editable && !!this.value;
    }
}
__decorate([
    property({ type: Boolean, reflect: true })
], FormFieldElement.prototype, "error", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], FormFieldElement.prototype, "warning", void 0);
__decorate([
    property({ type: String })
], FormFieldElement.prototype, "placeholder", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], FormFieldElement.prototype, "transparent", void 0);
__decorate([
    property({ type: Boolean })
], FormFieldElement.prototype, "clears", void 0);
__decorate([
    state()
], FormFieldElement.prototype, "inputAriaLabel", void 0);
__decorate([
    state()
], FormFieldElement.prototype, "inputAriaDescription", void 0);
__decorate([
    state()
], FormFieldElement.prototype, "inputAriaRequired", void 0);
