import { DuplicateElementError } from '../errors/DuplicateElementError.js';
import { ready } from '../utils/elementReady.js';
import { isLocalhost } from '../utils/helpers.js';
import { CustomStyleRegistry } from './CustomStyleRegistry.js';
class ElementRegistrationItem {
    constructor(definition) {
        this.creations = 0;
        this.connections = 0;
        this.disconnections = 0;
        this.version = definition.version;
    }
}
const register = new Map();
const upgrade = (name, definition) => {
    definition.applyThemeStyles(CustomStyleRegistry.get(name));
    const isElementDefined = customElements.get(name);
    // For localhost, define custom element even when it's been defined already to inform developers about the issue.
    if (isLocalhost || !isElementDefined) {
        customElements.define(name, definition);
    }
};
export class ElementRegistry {
    /**
     * Define a new custom element into the registry.
     * @param name tag name of the custom element
     * @param definition the class definition of the element
     * @param [options] element definition parameters
     * @returns {void}
     */
    static define(name, definition) {
        if (register.has(name) && isLocalhost) {
            // Allow the application to still load
            setTimeout(() => {
                throw new DuplicateElementError(name);
            });
        }
        else {
            const registrationItem = new ElementRegistrationItem(definition);
            register.set(name, registrationItem);
            ready(name, () => {
                upgrade(name, definition);
            });
        }
    }
    /**
     * Gets the definition of an already defined custom element.
     * @param name tag name of the custom element
     * @returns Element registration object, or, `undefined`,
     * when there is no item registered by the provided name.
     */
    static get(name) {
        return register.get(name);
    }
    /**
     * Logs the creation of an element
     * @param element Element to register the creation of
     * @returns {void}
     */
    static create(element) {
        const { localName: name } = element;
        if (register.has(name)) {
            register.get(name).creations += 1;
        }
    }
    /**
     * Logs the connection of an element
     * @param element Element to register the connection of
     * @returns {void}
     */
    static connect(element) {
        const { localName: name } = element;
        if (register.has(name)) {
            register.get(name).connections += 1;
        }
    }
    /**
     * Logs the disconnection of an element
     * @param element Element to register the disconnection of
     * @returns {void}
     */
    static disconnect(element) {
        const { localName: name } = element;
        if (register.has(name)) {
            register.get(name).disconnections += 1;
        }
    }
}
