import { isBasicElement } from '../utils/helpers.js';
const register = new Set(); /* Track all active elements */
const focusedMap = new Map(); /* Track all focused elements */
const FocusedPropertyKey = Symbol('focused');
let autoFocusFrame = null;
const autoFocus = (element) => {
    /* need to wait for elements to render, first come first focused */
    if (element.autofocus && autoFocusFrame === null) {
        autoFocusFrame = requestAnimationFrame(() => {
            autoFocusFrame = null;
            element.focus();
        });
    }
};
// Keys are unreliable when performed outside the main window
let isKey = false; /* true if any key is pressed */
let isKeyTab = false; /* true if Tab is pressed */
let isKeyShift = false; /* true if Shift is pressed */
/**
 * Reset keys. Run of keyup or on window blur
 * @returns {void}
 */
const resetKeys = () => {
    isKey = false;
    isKeyTab = false;
    isKeyShift = false;
};
/**
 * Get an active element either on document or inside the shadow root
 * @param [deep=false] set to true to look through all shadow root elements
 * @returns active element or null
 */
const getActiveElement = (deep = false) => {
    const getShadowActiveElement = (activeElement) => {
        if (activeElement?.shadowRoot?.activeElement) {
            if (deep) {
                return getShadowActiveElement(activeElement.shadowRoot.activeElement);
            }
            return activeElement.shadowRoot.activeElement;
        }
        return activeElement;
    };
    return getShadowActiveElement(document.activeElement);
};
/**
 * Delegate focus to the first or last tabbable element
 * @param element Custom element
 * @param [first=true] Set to false to delegate to last tabbable element
 * @returns {void}
 */
const delegateFocus = (element, first = true) => {
    const tabbableElements = element.tabbableElements;
    const delegateFocusElement = tabbableElements[first ? 0 : tabbableElements.length - 1];
    if (delegateFocusElement && delegateFocusElement !== element) {
        delegateFocusElement.focus();
    }
};
/**
 * Get all elements in ancestor tree that are of BasicElement
 * @param target A node to start searching from
 * @param [includeAll=false] Set to true to include all registered elements, not just tabbable
 * @returns collection of BasicElement
 */
const getRegisteredPath = (target, includeAll = false) => {
    let node = target;
    const elements = [];
    while (node) {
        if (isBasicElement(node) && register.has(node) && (includeAll || node.tabbable)) {
            elements.push(node);
        }
        if (node.nodeType === Node.DOCUMENT_FRAGMENT_NODE) {
            node = node.host;
        }
        else {
            node = node.parentNode;
        }
    }
    return elements;
};
/**
 * Run when document keydown event happens
 * Pass through the focus on shift tab, to delegate to the previous element
 * @param event Keyboard event
 * @returns {void}
 */
const onDocumentKeyDown = (event) => {
    isKey = true;
    isKeyTab = event.key === 'Tab';
    isKeyShift = event.shiftKey;
    if (isKeyTab && isKeyShift) {
        // If current focused is the first in focus, focus on self to move to previous element
        const path = getRegisteredPath(event.composedPath()[0]);
        const activeElement = getActiveElement(true);
        for (let i = path.length - 1; i >= 0; i -= 1) {
            const parent = path[i];
            const firstTabbableElement = parent.tabbableElements[0];
            if (firstTabbableElement === activeElement) {
                parent.focus();
                return;
            }
        }
    }
};
/**
 * Check if an element should delegate focus
 * @param target Target
 * @returns true if should
 */
const shouldDelegateOnFocus = (target) => {
    return (!isKeyShift &&
        isBasicElement(target) &&
        register.has(target) &&
        target.delegatesFocus &&
        getActiveElement(true) === target);
};
/**
 * Dispatch `focused-changed` event
 * @param element Element to dispatch event for
 * @param focused Focused state
 * @returns {void}
 */
const dispatchFocusedChangedEvent = (element, focused) => {
    const event = new CustomEvent('focused-changed', {
        detail: {
            value: focused
        }
    });
    element.dispatchEvent(event);
};
let syncAnimationFrame = null;
/**
 * Set element `focused` state in asynchronous way
 * @returns {void}
 */
const updateFocusedState = () => {
    if (syncAnimationFrame) {
        cancelAnimationFrame(syncAnimationFrame);
    }
    const focused = isKey ? 'visible' : '';
    syncAnimationFrame = requestAnimationFrame(() => {
        syncAnimationFrame = null;
        const activeElement = getActiveElement(true);
        const focusedPath = activeElement ? getRegisteredPath(activeElement, true) : [];
        focusedMap.forEach((state, el) => {
            if (!focusedPath.includes(el)) {
                focusedMap.delete(el);
                el.removeAttribute('focused');
                dispatchFocusedChangedEvent(el, false);
                el.requestUpdate(FocusedPropertyKey, true);
            }
        });
        focusedPath.forEach((el) => {
            const focusedChanged = !focusedMap.has(el);
            if (!focusedChanged || focusedMap.get(el) !== focused) {
                focusedMap.set(el, focused);
                el.setAttribute('focused', focused);
                dispatchFocusedChangedEvent(el, true);
                focusedChanged && el.requestUpdate(FocusedPropertyKey, false);
            }
        });
    });
};
/**
 * Run on focus event
 * @param event Focus event
 * @returns {void}
 */
const onFocus = (event) => {
    const target = event.target;
    shouldDelegateOnFocus(target) && delegateFocus(target);
    updateFocusedState();
};
/**
 * Run on blur event
 * @returns {void}
 */
const onBlur = () => {
    updateFocusedState();
};
const onDocumentFocus = (event) => onFocus(event);
const onDocumentBlur = () => onBlur();
const onDocumentKeyUp = () => resetKeys();
const onWindowBlur = () => resetKeys();
const onShadowRootFocus = function (event) {
    onFocus(event);
};
const onShadowRootBlur = function () {
    const element = this.host;
    /* this is required because Chrome does not fire focus event on document or shadowRoot when focus
    changes from shadowRoot to element itself */
    if (element.delegatesFocus) {
        requestAnimationFrame(() => {
            /* frame is required, as immediately after blur, activeElement is body */
            if (getActiveElement(true) === element) {
                delegateFocus(element);
            }
            updateFocusedState();
        });
    }
};
/**
 * add focus & blur event listeners to shadow roots
 * @param element an element which event listeners should be added to its shadow roots
 * @return {void}
 */
const addShadowRootEventListeners = (element) => {
    const RootNode = element.getRootNode();
    if (RootNode instanceof ShadowRoot) {
        // No need to worry about duplicate listeners as same callback with the same scope is skipped
        RootNode.addEventListener('focus', onShadowRootFocus, true);
        RootNode.addEventListener('blur', onShadowRootBlur, true);
        addShadowRootEventListeners(RootNode.host);
    }
};
class FocusRegistry {
    /**
     * Connect an element to focus global listener
     * @param element Element to register the connection of
     * @returns {void}
     */
    static connect(element) {
        if (!register.size) {
            document.addEventListener('keydown', onDocumentKeyDown, true);
            document.addEventListener('keyup', onDocumentKeyUp, true);
            document.addEventListener('focus', onDocumentFocus, true);
            document.addEventListener('blur', onDocumentBlur, true);
            window.addEventListener('blur', onWindowBlur);
        }
        addShadowRootEventListeners(element);
        register.add(element);
        autoFocus(element);
    }
    /**
     * Disconnect an element of focus global listener
     * @param element Element to register the disconnection of
     * @returns {void}
     */
    static disconnect(element) {
        register.delete(element);
        if (!register.size) {
            document.removeEventListener('keydown', onDocumentKeyDown, true);
            document.removeEventListener('keyup', onDocumentKeyUp, true);
            document.removeEventListener('focus', onDocumentFocus, true);
            document.removeEventListener('blur', onDocumentBlur, true);
            window.removeEventListener('blur', onWindowBlur);
        }
    }
}
export { FocusedPropertyKey, FocusRegistry };
