/**
 * An improved copy of IronFocusablesHelper
 */
export declare abstract class FocusableHelper {
    /**
     * Returns the normalized element tabindex. If not focusable, returns -1.
     * It checks for the attribute "tabindex" instead of the element property
     * `tabIndex` since browsers assign different values to it.
     * e.g. in Firefox `<div contenteditable>` has `tabIndex = -1`
     * @param element Element
     * @returns normalizedTabIndex
     */
    private static normalizedTabIndex;
    /**
     * Searches for nodes that are tabbable and adds them to the `result` array.
     * Returns if the `result` array needs to be sorted by tabindex.
     * @param node The starting point for the search; added to `result` if tabbable.
     * @param result Result
     * @param delegatedList A collection of nodes, which cannot follow DOM order
     * @returns needsSort
     */
    private static collectTabbableNodes;
    /**
     * Sorts an array of tabbable elements by tabindex. Returns a new array.
     * @param tabbables Tabbables
     * @returns sortedList
     */
    private static sortByTabIndex;
    /**
     * Process remaining nodes to compose the list of tabbable nodes taking into account that
     * some elements are delegated
     * @param result Result
     * @param delegatedList A collection of nodes, which cannot follow natural DOM order
     * @returns {void}
     */
    private static composeDelegated;
    /**
     * Merge sort iterator, merges the two arrays into one, sorted by tab index.
     * @param left Left list
     * @param right Right list
     * @returns sorted list
     */
    private static mergeSortByTabIndex;
    /**
     * Returns if element `a` has lower tab order compared to element `b`
     * (both elements are assumed to be focusable and tabbable).
     * Elements with tabindex = 0 have lower tab order compared to elements
     * with tabindex > 0.
     * If both have same tabindex, it returns false.
     * @param a Left-side element
     * @param b Right-side element
     * @returns isLower
     */
    private static hasLowerTabOrder;
    /**
     * Returns false if the element has `visibility: hidden` or `display: none`
     * @param element Element
     * @returns visible
     */
    private static isVisible;
    /**
     * Returns a sorted array of tabbable nodes, including the root node.
     * It searches the tabbable nodes in the light and shadow dom of the children,
     * sorting the result by tabindex.
     * @param node Node
     * @returns tabbableNodes
     */
    static getTabbableNodes(node: Node): HTMLElement[];
    /**
     * Returns if a element is focusable.
     * @param element Element
     * @returns focusable
     */
    static isFocusable(element: HTMLElement): boolean;
    /**
     * Returns if a element is tabbable. To be tabbable, a element must be
     * focusable, visible, and with a tabindex !== -1.
     * @param element Element
     * @returns tabbable
     */
    static isTabbable(element: HTMLElement): boolean;
}
